'use strict';

var path = require('path');
var fs = require('fs');

/**
 * @description 插件名称
 */
const PLUGIN_NAME = 'vue-cli-plugin-inject-alias';
/**
 * @description 处理插件默认配置
 */
const DEFAULT_CONFIG = {
    root: path.join(process.cwd(), 'src'),
    prefix: '@',
    mode: 'all'
};

/**
 * @description 是否是文件夹
 * @param path
 * @returns
 */
const isDir = (path) => {
    return fs.lstatSync(path).isDirectory();
};
/**
 * @description 是否存在文件
 * @param path
 * @returns
 */
const hasFile = (path) => {
    return fs.existsSync(path);
};
/**
 * @description 获取所有文件夹
 * @param path
 * @returns
 */
const getDirs = (path$1) => {
    const dirs = fs.readdirSync(path$1);
    return dirs.reduce((result, name) => {
        const fullPath = path.join(path$1, name);
        isDir(fullPath) && result.push({ dirName: name, dirPath: fullPath });
        return result;
    }, []);
};
/**
 * @description 合并配置
 * @param source object
 * @param target object
 * @returns object
 */
function mergeConfig(source, target) {
    const isObject = (data) => Object.prototype.toString.call(data) === '[object Object]';
    const hasKey = (data, key) => isObject(data) && Reflect.has(data, key);
    for (const key in target) {
        if (!hasKey(source, key)) {
            source[key] = target[key];
            isObject(target[key]) && mergeConfig(source[key], target[key]);
        }
        else {
            isObject(target[key]) && mergeConfig(source[key], target[key]);
        }
    }
    return source;
}
/**
 * @description 处理插件默认配置
 * @param root string
 */
const getPluginConfig = root => {
    return {
        [PLUGIN_NAME]: {
            root: path.join(root, 'src'),
            prefix: DEFAULT_CONFIG.prefix,
            mode: DEFAULT_CONFIG.mode
        }
    };
};

/**
 * @description 获取json
 * @param jsonPath jsonPath
 * @returns
 */
function getJson(jsonPath) {
    try {
        const jsonText = fs.readFileSync(jsonPath, 'utf-8');
        return JSON.parse(jsonText);
    }
    catch (error) {
        process.exit(0);
    }
}
/**
 * @description 合并json
 * @param target IJson
 * @param source IJson
 * @returns IJson
 */
function mergeJson(target, source) {
    const targetPaths = target.compilerOptions?.paths ?? {};
    const sourcePaths = source.compilerOptions?.paths ?? {};
    for (const pathKey in targetPaths) {
        if (!Reflect.has(sourcePaths, pathKey)) {
            sourcePaths[pathKey] = targetPaths[pathKey];
        }
    }
    return {
        ...source,
        compilerOptions: {
            ...source.compilerOptions,
            paths: sourcePaths
        }
    };
}
/**
 * @description 生成json
 * @param alias { [key: string]: string }
 * @param root string
 * @param prefix string
 * @returns IJson
 */
function genJson(alias, root, prefix) {
    const { name: rootName } = path.parse(root);
    const paths = Object.keys(alias).reduce((result, pathKey) => {
        if (pathKey.startsWith(prefix) && pathKey !== prefix) {
            const { name } = path.parse(alias[pathKey]);
            result[`${pathKey}/*`] = [`${rootName}/${name}/*`];
        }
        result[`${prefix}/*`] = [`${rootName}/*`];
        return result;
    }, {});
    return {
        compilerOptions: {
            paths: paths,
            baseUrl: './'
        }
    };
}
/**
 * @description 同步json文件
 * @param extendJson 继承的json
 * @param jsJson jsconfig.json
 * @param tsJson tsconfig.json
 */
function syncJson({ extendJson, jsJson, tsJson, alias, prefix, root, mode }) {
    if (hasFile(extendJson) && ['all', 'extends'].includes(mode)) {
        const json = genJson(alias, root, prefix);
        hasFile(extendJson) && fs.writeFileSync(extendJson, JSON.stringify(json, null, 4));
    }
    if (hasFile(jsJson) && ['all', 'sync'].includes(mode)) {
        const target = genJson(alias, root, prefix);
        const source = getJson(jsJson);
        const newJson = mergeJson(target, source);
        hasFile(jsJson) && fs.writeFileSync(jsJson, JSON.stringify(newJson, null, 4));
    }
    if (hasFile(tsJson) && ['all', 'sync'].includes(mode)) {
        const target = genJson(alias, root, prefix);
        const source = getJson(tsJson);
        const newJson = mergeJson(target, source);
        hasFile(tsJson) && fs.writeFileSync(tsJson, JSON.stringify(newJson, null, 4));
    }
}

const ALIAS_JSON_PATH = (cwd) => path.resolve(cwd, 'node_modules/@jiangweiye/tsconfig/tsconfig.alias.json');
const jsconfig = (root) => path.join(root, 'jsconfig.json');
const tsconfig = (root) => path.join(root, 'tsconfig.json');
function genAlias(root, prefix) {
    const dirs = getDirs(root);
    return dirs.reduce((result, item) => {
        const key = `${prefix}${item.dirName}`;
        const value = item.dirPath;
        result[key] = value;
        return result;
    }, {});
}
function alias(api, options) {
    const { pluginOptions = {} } = options;
    const cwd = api.getCwd();
    // 插件配置
    const { mode, prefix, root } = mergeConfig(pluginOptions, getPluginConfig(cwd))[PLUGIN_NAME];
    if (!hasFile(root)) {
        return;
    }
    api.configureWebpack(config => {
        let baseAlias = {};
        if (config.resolve && config.resolve.alias) {
            baseAlias = config.resolve.alias;
        }
        const _alias = { ...genAlias(root, prefix), ...baseAlias };
        syncJson({
            extendJson: ALIAS_JSON_PATH(cwd),
            jsJson: jsconfig(cwd),
            tsJson: tsconfig(cwd),
            alias: _alias,
            root,
            prefix,
            mode
        });
        return {
            resolve: {
                alias: _alias
            }
        };
    });
}

module.exports = alias;
